/*
 * apt-get install apparmor-dev
 * gcc -o test-1562989 main.c -lapparmor
 *
 * reproducer for the bug:
 * $ sudo apparmor_parser -r ./app-profile ./launcher-profile
 * $ gcc -o test-1562989 main.c -lapparmor
 * $ sudo chown root:root ./test-1562989
 * $ sudo chmod 4755 ./test-1562989
 * $ aa-exec -p launcher -- ./test-1562989 ubuntu-clock-app.ubuntucoredev_clock_3.6+snap2 /usr/bin/uptime
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <string.h>
#include <stdint.h>
#include <inttypes.h>
#include <fcntl.h>

#include <errno.h>
#include <stdarg.h>
#include <sys/mount.h>
#include <sys/apparmor.h>
#include <sched.h>

void die(const char *msg, ...)
{
	va_list va;
	va_start(va, msg);
	vfprintf(stderr, msg, va);
	va_end(va);

	if (errno != 0) {
		perror(". errmsg");
	} else {
		fprintf(stderr, "\n");
	}
	exit(1);
}

int main(int argc, char *argv[])
{
	int i;
	int rc = 0;
	unsigned real_uid = getuid();
	unsigned real_gid = getgid();
	if (argc < 3) {
		fprintf(stderr, "Usage: %s <profile> <cmd>\n", argv[0]);
		return 1;
	}
	for (i = 0; i < argc; i++)
		printf("argv[%d]: %s\n", i, argv[i]);

	rc = aa_change_onexec(argv[1]);
	if (rc != 0) {
		die("aa_change_onexec failed with %i", rc);
	}
	execv(argv[2], &argv[2]);

	return 0;
}
